@@ -8,7 +8,7 @@ use MarpaX::Languages::M4::Impl::Default;
 class MarpaX::Languages::M4 {
     extends 'MarpaX::Languages::M4::Impl::Default';
 
-    our $VERSION = '0.016'; # VERSION
+    our $VERSION = '0.017'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -28,13 +28,15 @@ class MarpaX::Languages::M4 {
     method raiseException (@args)   { $self->impl_raiseException(@args) }
     method file (@args)             { $self->impl_file(@args) }
     method line (@args)             { $self->impl_line(@args) }
-    method debugfile (@args)        { $self->impl_debugfile(@args) }
     method rc (@args)               { $self->impl_rc(@args) }
     method isImplException (@args)  { $self->impl_isImplException(@args) }
     method macroExecute (@args)     { $self->impl_macroExecute(@args) }
     method macroCallId (@args)      { $self->impl_macroCallId(@args) }
     method nbInputProcessed (@args) { $self->impl_nbInputProcessed(@args) }
     method readFromStdin (@args)    { $self->impl_readFromStdin(@args) }
+    method canLog (@args)           { $self->impl_canLog(@args) }
+    method debugFile (@args)        { $self->impl_debugFile(@args) }
+    method defaultWarnMacroSequence (ClassName $class: @args) { $class->impl_defaultWarnMacroSequence(@args) }
 }
 
 
@@ -52,7 +54,7 @@ MarpaX::Languages::M4 - M4 pre-processor
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 SYNOPSIS
 
@@ -136,10 +138,6 @@ Get current file name. See NOTES.
 
 Get current line number. See NOTES.
 
-=head2 $self->debugfile(--> Undef|Str)
-
-Get current debug file name.
-
 =head2 $self->rc(--> Int)
 
 Get current parse return code. Should be POSIX::EXIT_SUCCESS() or POSIX::EXIT_FAILURE().
@@ -164,13 +162,23 @@ Return number of input processed so far.
 
 Enters interactive mode.
 
+=head2 $self->debugFile(--> Undef|Str)
+
+Return debug file, undef if none.
+
+=head2 $class->defaultWarnMacroSequence(--> Str)
+
+Return default --warn-macro-sequence option value (used by m4pp to handle MooX::Option non support of optional value on the command-line)
+
 =head1 NOTES
 
 file() and line() methods, nor synchronisation output, are currently not supported. This is on the TODO list for this package.
 
+M4 is a MooX::Role::Logger consumer, using explicitely Log::Any's "f" functions in all its logging methods. This mean that the message, in case you would use something like e.g. Log::Any::Adapter::Callback, is always formatted, with no additional parameter.
+
 =head1 SEE ALSO
 
-L<Marpa::R2>, L<Moops>, L<POSIX>, L<M4 POSIX|http://pubs.opengroup.org/onlinepubs/9699919799/utilities/m4.html>, L<M4 GNU|https://www.gnu.org/software/m4/manual/m4.html>
+L<Marpa::R2>, L<Moops>, L<MooX::Role::Logger>, L<Log::Any>, L<Log::Any::Adapter::Callback>, L<POSIX>, L<M4 POSIX|http://pubs.opengroup.org/onlinepubs/9699919799/utilities/m4.html>, L<M4 GNU|https://www.gnu.org/software/m4/manual/m4.html>
 
 =head1 AUTHOR
 
