@@ -6,7 +6,7 @@ use Moops;
 
 class MarpaX::Languages::M4::Impl::Macro {
 
-    our $VERSION = '0.016'; # VERSION
+    our $VERSION = '0.017'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -29,7 +29,7 @@ class MarpaX::Languages::M4::Impl::Macro {
     );
     has expansion => (
         is       => 'rw',
-        isa      => Str | M4Macro,
+        isa      => Undef | Str,
         required => 1
     );
     has needParams => (
@@ -46,7 +46,12 @@ class MarpaX::Languages::M4::Impl::Macro {
         handles_via => 'Hash',
         handles     => {
             _paramCanBeMacro_get    => 'get',
-            _paramCanBeMacro_exists => 'exists'
+            _paramCanBeMacro_exists => 'exists',
+            #
+            # We do not hold references in values, so shallow_clone
+            # is a real clone
+            #
+            _paramCanBeMacro_clone => 'shallow_clone'
         }
     );
     has postMatchLength => (
@@ -63,7 +68,7 @@ class MarpaX::Languages::M4::Impl::Macro {
         return $self->name;
     }
 
-    method macro_expansion (--> Str | M4Macro) {
+    method macro_expansion (--> Undef | Str) {
         return $self->expansion;
     }
 
@@ -72,7 +77,7 @@ class MarpaX::Languages::M4::Impl::Macro {
     }
 
     method macro_isBuiltin (--> Bool) {
-        return ( $self->expansion == $self ) ? true : false;
+        return Undef->check( $self->expansion );
     }
 
     method macro_paramCanBeMacro (PositiveOrZeroInt $paramPos --> Bool) {
@@ -90,6 +95,25 @@ class MarpaX::Languages::M4::Impl::Macro {
         }
     }
 
+    method macro_clone (Str $name --> M4Macro) {
+        my $macro = __PACKAGE__->new(
+            name       => $name,
+            stub       => $self->stub,
+            needParams => $self->needParams,
+            #
+            # Cloning a builtin is a builtin
+            #
+            expansion => $self->expansion,
+            #
+            # No need of clone: the hash is ro once created
+            #
+            paramCanBeMacro => $self->_paramCanBeMacro_clone,
+            postMatchLength => $self->postMatchLength
+        );
+
+        return $macro;
+    }
+
     with 'MarpaX::Languages::M4::Role::Macro';
 }
 
@@ -107,7 +131,7 @@ MarpaX::Languages::M4::Impl::Macro - M4 Macro generic implementation
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
