@@ -15,8 +15,12 @@ class MarpaX::Languages::M4::Impl::Parser {
     use Marpa::R2;                               # 2.103_004;
     use Scalar::Util qw/readonly/;
     use Types::Common::Numeric -all;
+    #
+    # --nesting_limit (SHOULD) take care of that
+    #
+    no warnings 'recursion';
 
-    our $VERSION = '0.016'; # VERSION
+    our $VERSION = '0.017'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -105,12 +109,85 @@ COMMA ~ ',' _WS_any
         );
     };
 
+    BEGIN {
+      #
+      # Marpa internal optimisation: we do not want the closures to be rechecked every time
+      # we call $r->value(). This is a static information, although determined at run-time
+      # the first time $r->value() is called on a recognizer.
+      #
+      no warnings 'redefine';
+
+      sub Marpa::R2::Recognizer::registrations {
+        my $recce = shift;
+        if (@_) {
+          my $hash = shift;
+          if (! defined($hash) ||
+              ref($hash) ne 'HASH' ||
+              grep {! exists($hash->{$_})} qw/
+                                               NULL_VALUES
+                                               REGISTRATIONS
+                                               CLOSURE_BY_SYMBOL_ID
+                                               CLOSURE_BY_RULE_ID
+                                               RESOLVE_PACKAGE
+                                               RESOLVE_PACKAGE_SOURCE
+                                               PER_PARSE_CONSTRUCTOR
+                                             /) {
+            Marpa::R2::exception(
+                                 "Attempt to reuse registrations failed:\n",
+                                 "  Registration data is not a hash containing all necessary keys:\n",
+                                 "  Got : " . ((ref($hash) eq 'HASH') ? join(', ', sort keys %{$hash}) : '') . "\n",
+                                 "  Want: CLOSURE_BY_RULE_ID, CLOSURE_BY_SYMBOL_ID, NULL_VALUES, PER_PARSE_CONSTRUCTOR, REGISTRATIONS, RESOLVE_PACKAGE, RESOLVE_PACKAGE_SOURCE\n"
+                                );
+          }
+          $recce->[Marpa::R2::Internal::Recognizer::NULL_VALUES] = $hash->{NULL_VALUES};
+          $recce->[Marpa::R2::Internal::Recognizer::REGISTRATIONS] = $hash->{REGISTRATIONS};
+          $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_SYMBOL_ID] = $hash->{CLOSURE_BY_SYMBOL_ID};
+          $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_RULE_ID] = $hash->{CLOSURE_BY_RULE_ID};
+          $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE] = $hash->{RESOLVE_PACKAGE};
+          $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE] = $hash->{RESOLVE_PACKAGE_SOURCE};
+          $recce->[Marpa::R2::Internal::Recognizer::PER_PARSE_CONSTRUCTOR] = $hash->{PER_PARSE_CONSTRUCTOR};
+        }
+        return {
+                NULL_VALUES => $recce->[Marpa::R2::Internal::Recognizer::NULL_VALUES],
+                REGISTRATIONS => $recce->[Marpa::R2::Internal::Recognizer::REGISTRATIONS],
+                CLOSURE_BY_SYMBOL_ID => $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_SYMBOL_ID],
+                CLOSURE_BY_RULE_ID => $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_RULE_ID],
+                RESOLVE_PACKAGE => $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE],
+                RESOLVE_PACKAGE_SOURCE => $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE],
+                PER_PARSE_CONSTRUCTOR => $recce->[Marpa::R2::Internal::Recognizer::PER_PARSE_CONSTRUCTOR]
+               };
+      } ## end sub registrations
+
+      sub Marpa::R2::Scanless::R::registrations {
+        my $slr = shift;
+        my $thick_g1_recce =
+          $slr->[Marpa::R2::Internal::Scanless::R::THICK_G1_RECCE];
+        return $thick_g1_recce->registrations(@_);
+      } ## end sub Marpa::R2::Scanless::R::registrations
+
+    }
+
     has _parse_level => (
-        is      => 'rwp',
-        isa     => PositiveOrZeroInt,
+        is  => 'rwp',
+        isa => PositiveOrZeroInt,
+
+        # trigger => 1,
         default => 0
     );
 
+    method _trigger__parse_level (PositiveOrZeroInt $parse_level, @rest) {
+            #
+            # GNU testing of nesting_limit is another implementation, this
+            # has no meaning for us. So, even if supported, this option does
+            # nothing: the trigger is NOT enabled.
+            #
+        my $nesting_limit = $self->impl_nestingLimit;
+        if ( $nesting_limit > 0 && $parse_level > $nesting_limit ) {
+            $self->impl_raiseException(
+                sprintf( 'Nesting limit of %d exceeded', $nesting_limit ) );
+        }
+    }
+
     method parser_parse (Str $input --> Str) {
             #
             # We will modify the buffer in-place, therefore we want to
@@ -157,8 +234,7 @@ COMMA ~ ',' _WS_any
         #
         my $r = Marpa::R2::Scanless::R->new(
             {   grammar => $g,
-                semantics_package =>
-                    sprintf( '%s::%s', __PACKAGE__, 'Actions' ),
+                semantics_package => 'MarpaX::Languages::M4::Impl::Parser::Actions',
                 exhaustion => 'event',
 
                 # trace_terminals => 1,
@@ -252,11 +328,14 @@ COMMA ~ ',' _WS_any
                 elsif ( $_ eq 'LPAREN' ) {
                     if (   exists( $expected{LPAREN} )
                         && $canCollectArguments
-                        && substr( ${$inputRef}, $rc{pos}, 1 ) eq '(' )
+                        && do { pos( ${$inputRef} ) = $rc{pos}; 1 }
+                        && ${$inputRef} =~ /\G\(\s*/
+                        )
                     {
                         $lexeme       = 'LPAREN';
-                        $lexemeValue  = '(';
-                        $lexemeLength = 1;
+                        $lexemeLength = $+[0] - $-[0];
+                        $lexemeValue
+                            = substr( ${$inputRef}, $-[0], $lexemeLength );
                         last;
                     }
                 }
@@ -350,6 +429,10 @@ COMMA ~ ',' _WS_any
                     )
                     )
                 {
+                    my $canTrace = $self->_canTrace($thisMacro);
+                    my $macroCallId
+                        = $self->impl_macroExecuteHeader( $thisMacro,
+                        $canTrace );
                     #
                     # Collect macro arguments
                     #
@@ -374,7 +457,8 @@ COMMA ~ ',' _WS_any
                     # that a macro never croaks.
                     #
                     $lexemeValue
-                        = $self->impl_macroExecute( $thisMacro, @args );
+                        = $self->impl_macroExecuteNoHeader( $thisMacro,
+                        $macroCallId, $canTrace, @args );
                     #
                     # Eventual postmatch length
                     #
@@ -473,7 +557,18 @@ COMMA ~ ',' _WS_any
             # is of concern for a macro that called us.
             #
             local $MarpaX::Languages::M4::Impl::Parser::macro = $macro;
+            #
+            # For $r->value() optimisations
+            #
+            state $registrations = undef;
+
+            if (defined($registrations)) {
+              $r->registrations($registrations);
+            }
             my $valueRef = $r->value;
+            if (! defined($registrations)) {
+              $registrations = $r->registrations();
+            }
             if ( Undef->check($valueRef) || Undef->check( ${$valueRef} ) ) {
                 goto return_error;
             }
@@ -540,7 +635,7 @@ MarpaX::Languages::M4::Impl::Parser - M4 Marpa parser
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
