@@ -6,7 +6,7 @@ use Moops;
 
 role MarpaX::Languages::M4::Role::Impl {
 
-    our $VERSION = '0.016'; # VERSION
+    our $VERSION = '0.017'; # VERSION
 
     our $AUTHORITY = 'cpan:JDDPAUSE'; # AUTHORITY
 
@@ -20,22 +20,28 @@ role MarpaX::Languages::M4::Role::Impl {
     requires 'impl_value';
     requires 'impl_valueRef';
     requires 'impl_parseIncremental';
+    requires 'impl_parseIncrementalFile';
     requires 'impl_parse';
     requires 'impl_unparsed';
     requires 'impl_setEoi';
     requires 'impl_eoi';
     requires 'impl_raiseException';
+    requires 'impl_program';
+    requires 'impl_file';
     requires 'impl_line';
-    requires 'impl_debugfile';
     requires 'impl_rc';
     requires 'impl_isImplException';
+    requires 'impl_macroExecuteHeader';
+    requires 'impl_macroExecuteNoHeader';
     requires 'impl_macroExecute';
     requires 'impl_macroCallId';
-    requires 'impl_debugfile';
     requires 'impl_reloadState';
     requires 'impl_freezeState';
     requires 'impl_nbInputProcessed';
     requires 'impl_readFromStdin';
+    requires 'impl_canLog';
+    requires 'impl_debugFile';
+    requires 'impl_nestingLimit';
 
     with 'MarpaX::Languages::M4::Role::Builtin';
     with 'MarpaX::Languages::M4::Role::Logger';
@@ -56,7 +62,7 @@ MarpaX::Languages::M4::Role::Impl - M4 implementation role
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
